"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Motion = void 0;

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var Motion = _createClass(function Motion(visual, params) {
  var _this = this;

  _classCallCheck(this, Motion);

  _defineProperty(this, "update", function () {
    var axes = null;
    var _this$manager = _this.manager,
        _this$manager$contain = _this$manager.container,
        offsetWidth = _this$manager$contain.offsetWidth,
        offsetHeight = _this$manager$contain.offsetHeight,
        mouse = _this$manager.mouse,
        touch = _this$manager.touch;

    for (var m in _this.modes) {
      switch (m) {
        case 'trackMouse':
          var trackMouseX = _this.modes[m].mouseX;
          var trackMouseY = _this.modes[m].mouseY;

          if (trackMouseX) {
            _this.trackMouseByAxis({
              track: trackMouseX,
              axis: trackMouseX.symmetry ? mouse.x - offsetWidth / 2 : mouse.x
            });
          }

          if (trackMouseY) {
            _this.trackMouseByAxis({
              track: trackMouseY,
              axis: trackMouseY.symmetry ? mouse.y - offsetHeight / 2 : mouse.y
            });
          }

          break;

        case 'trackTouch':
          var trackTouchX = _this.modes[m].touchX;
          var trackTouchY = _this.modes[m].touchY;

          if (trackTouchX) {
            _this.trackMouseByAxis({
              track: trackTouchX,
              axis: trackTouchX.symmetry ? touch.x - offsetWidth / 2 : touch.x
            });
          }

          if (trackTouchY) {
            _this.trackMouseByAxis({
              track: trackTouchY,
              axis: trackTouchY.symmetry ? touch.y - offsetHeight / 2 : touch.y
            });
          }

          break;

        case 'uniforms':
          var uniforms = _this.visual.material.uniforms;

          if (uniforms) {
            for (var u in _this.modes[m]) {
              uniforms[u].value += _this.modes[m][u];
            }
          }

          break;

        case 'morph':
          var mti = _this.visual.morphTargetInfluences;

          for (var i = 0; i < mti.length; i++) {
            mti[i] += _this.modes[m].step;
          }

          break;

        case 'swarm':
          var _this$visual$geometry = _this.visual.geometry.attributes.position,
              array = _this$visual$geometry.array,
              count = _this$visual$geometry.count;
          axes = ['x', 'y', 'z'];

          for (var c = 0; c < count; c++) {
            var velocity = _this.swarmParams[c].velocity;

            for (var a = 0; a < axes.length; a++) {
              array[c * 3 + a] += velocity[axes[a]];

              if (array[c * 3 + a] < -_this.modes[m].maxValue || array[c * 3 + a] > _this.modes[m].maxValue) {
                velocity[axes[a]] = -velocity[axes[a]];
              }
            }
          }

          break;

        default:
          axes = _this.modes[m].axes;

          for (var _a in axes) {
            axes[_a].relativeValue += axes[_a].velocity;
            _this.visual[m][_a] += axes[_a].velocity;

            if (axes[_a].relativeValue < -axes[_a].maxValue || axes[_a].relativeValue > axes[_a].maxValue) {
              axes[_a].velocity = -axes[_a].velocity;
            }
          }

          break;
      }
    }
  });

  _defineProperty(this, "trackMouseByAxis", function (_ref) {
    var track = _ref.track,
        axis = _ref.axis;
    ['position', 'rotation', 'scale'].forEach(function (type) {
      if (track[type]) {
        if (track[type].x) {
          _this.visual[type].x = axis * track[type].x + (track[type].modX || 0);
        }

        if (track[type].y) {
          _this.visual[type].y = axis * track[type].y + (track[type].modY || 0);
        }

        if (track[type].z) {
          _this.visual[type].z = axis * track[type].z + (track[type].modZ || 0);
        }
      }
    });
  });

  this.visual = visual;
  this.modes = {};

  var _loop = function _loop(m) {
    if (m === 'swarm') {
      _this.swarmParams = [];
      var count = _this.visual.geometry.attributes.position.count;

      var _loop2 = function _loop2(c) {
        var velocity = {};
        ['x', 'y', 'z'].forEach(function (a) {
          var v = params[m].randVelocity;
          velocity[a] = params[m].axes.includes(a) ? -v + Math.random() * v * 2 : 0;
        });

        _this.swarmParams.push({
          velocity: velocity
        });
      };

      for (var c = 0; c < count; c++) {
        _loop2(c);
      }

      _this.modes[m] = params[m];
    } else if (params[m].axes) {
      var axes = params[m].axes.split('');
      _this.modes[m] = {
        axes: {}
      };
      axes.forEach(function (a) {
        return _this.modes[m].axes[a] = {
          relativeValue: 0,
          maxValue: params[m].maxValue,
          velocity: params[m].velocity ? params[m].reverse ? -params[m].velocity : params[m].velocity : -params[m].randVelocity + Math.random() * params[m].randVelocity * 2
        };
      });
    } else {
      _this.modes[m] = params[m];
    }
  };

  for (var m in params) {
    _loop(m);
  }
});

exports.Motion = Motion;